function M = Mchol( A,tol )

% tol=1e-3;  M = Mchol( A,tol );
% creates a sparse positive definite preconditioner M
% sets A2 = sparsified A (deleting off-diagonals smaller than tol)
% and returns M = decomposition(A2).
% With precon = 1, M maybe used as a preconditioner with minres
% for solving Ax = b.

% On entry:
% A       is a scaled symmetric positive-definite matrix (use scaleSPD( A )).
% tol     is the tolerance for creating A2 = sparsified A.

% 07 May 2020: First version of Mchol.
% Maintainers: Shaked Regev and Michael Saunders, ICME, Stanford University.

  n       = length(A);
  nnzA    = nnz(A);
  d       = diag(A);      % sparse column vector
  A       = tril(A,-1);   % Lower triangular part of A, excluding diagonal
  [I,J,V] = find(A);
  small   = find(abs(V) <= tol);
  nsmall  = 2*length(small);   % L + L'
  if nsmall>0
     I(small) = [];
     J(small) = [];
     V(small) = [];
     A = sparse(I,J,V,n,n);
  end
  A = A + A' + diag(d);   % exactly symmetric
  fprintf('\n nnz(A) = %9i      n   = %9i'  , nnzA,n)
  fprintf('\n nnz(A2)= %9i', nnz(A))
  fprintf('\n nsmall = %9i      tol = %9.1e', nsmall,tol)

  [L,p,s] = chol(A,'lower','vector');   % s is a permutation row vector
  fprintf('\n nnz(L) = %9i      from chol(A2)', nnz(L))
  clear L
  if p==0
      % Successful factorization: A2(s,s) = L*L'
      fprintf('\n Forming M using decomposition\n')
      M = decomposition(A,'chol');
  else
      fprintf('\n chol(A2) failed with p = %g.  Use smaller tol.  No M generated\n', p)
      M = [];
  end

end % function Mchol
