function M = SSAI(A,n,lfil)

%        M = SSAI(A,n,lfil);
% constructs a SPAI-type preconditioner from SPD A.
% Typically, lfil = ceil(nnz(A)/n);  = average nnz in a column of A.
% M is symmetric but might not be SPD.

% Shaked Regev and Michael Saunders
% ICME, Stanford University.

% 15 Aug 2019: Shaked's code entire_r_sparse_inverse.m from this date,
%              called from Shaked's SSAI.m.
%              A is assumed to have unit diagonals.
% 21 Apr 2020: Reformatted, renamed.  Trying it with Minres.
% 30 Apr 2020: Included in http://stanford.edu/group/SOL/software/minres/minres20.zip.
% 07 May 2020: tol added to exit loop if r is essentially 0.
%              Assumes A is scaled so norm(A) = O(1).
% 07 May 2020: Output M is exactly symmetric.
% 07 May 2020: Name changed from MSSAI to SSAI.

  M     = spalloc(n,n,lfil*n);
  itmax = 2*lfil;
  tol   = 1e-3;

  for j=1:n
     m = spalloc(n,1,lfil);
     r = sparse(j,1,1,n,1);

     for k=1:itmax
         [delta,i] = max(abs(r));
       % if delta<=tol, break, end
       % delta = r(i)/A(i,i);   % General
         delta = full(r(i));    % Assume A(i,i)=1
         m(i)  = m(i) + delta;
         if nnz(m)>=lfil, break; end
         r = r - delta*A(:,i);
     end
          
     M(:,j) = m;
  end

  d = diag(M);           % sparse column vector
  M = (M+M')*0.5;
  M = tril(M,-1);
  M = M + M' + diag(d);  % M is exactly symmetric
  fprintf('\n M symmetrized.  nnz(M) = %d\n', nnz(M))

end % function SSAI
