function xMldl = minresMldl( A,b,tol,rtol );

% tol=1e-3;  rtol=1e-8;  xMldl = minresMldl( A,b,tol,rtol );
% runs minres on Ax = b with preconditioner M = Mldl(A,tol)
% and accuracy tolerance rtol.
% Mldl makes A2 = sparsified A by deleting nonzeros smaller than tol.
% A can be symmetric positive-definite or indefinite.
% A should be scaled first (use scaleSID).

% Needs Mldl, MLBL, minres.

% 12 May 2020: First version of minresMldl.
% Maintainers: Shaked Regev and Michael Saunders, ICME, Stanford University.
% 16 Jun 2020: Added input rtol and output xMldl.

   fprintf('\n M = Mldl( A,tol,thresh )')
   tic
   n      = length(A);
   thresh = 0.01;       % thresh is in [0,0.5] (default 0.01)
   [L,D,p]= Mldl( A,tol,thresh );   % D has 1x1 or 2x2 blocks
   TMldl  = toc;
   fprintf(' Time for Mldl =%9.2e\n', TMldl)

   x0     = [];
   M      = @(x) MLBL( L,D,p,x );   % Function for solving L*B*L'y = x
   precon = 1;
   shift  = 0.0;
   show   = 1;
   check  = 0;
   itnlim = min(4*n,50000);
%  rtol   = 1e-8;   % Now an input parameter
   tolMPD = 1e-6;
   facMPD = 100;
   tic

   [ xMldl, istop, itn, rnorm, Arnorm, Anorm, Acond, xnorm, betacheck ] = ...
       minres( x0, A, b, M, precon, shift, show, check, itnlim, rtol, tolMPD );
   TSMldl = toc;
   fprintf(' Time for minres with Mldl =%9.2e\n', TSMldl)

end % function minresMldl
