function xminres2 = minresMminres2( A,b,rtol );

% rtol = 1e-8;   xminres2 = minresMminres2( A,b,rtol );
% runs minres on Ax = b with preconditioner Mminres2 and accuracy tolerance rtol.

% 29 Apr 2020: First version of minresMminres2.
%              Michael Saunders,
%              ICME, Stanford University.
% 09 May 2020: facMPD = 200.
%              Final preconditioner is M + gammasum*I.
% 16 Jun 2020: Input rtol and output xminres2 are helpful for testSID( rtol ).

   fprintf('\n M = Mminres2( A )')
   tic
   n      = length(A);
   M      = Mminres2( A );
   TMminres2 = toc;
   fprintf(' Time for Mminres2 =%9.2e\n', TMminres2)

   x0     = [];
   shift  = 0.0;
   show   = 1;
   check  = 0;
   itnlim = min(4*n,50000);
%  rtol   = 1e-8;   % Now an input parameter
   tolMPD = 1e-4;
   facMPD = 200;
   precon = 2;      % minres handles precon = 0,1,2
   tic

   [ xminres2, istop, itn, rnorm, Arnorm, Anorm, Acond, xnorm, betacheck ] = ...
       minres( x0, A, b, M, precon, shift, show, check, itnlim, rtol, tolMPD );

   itntotal   = itn;
   gammasum   = 0;
   nrestart   = 0;
   maxrestart = 50;

   while nrestart < maxrestart & istop==9
      nrestart = nrestart + 1;
      gamma    = facMPD*(tolMPD - betacheck);
      gammasum = gammasum + gamma;
      M        = M + gamma*speye(n);
      fprintf('\n nrestart =%3i         M = M + gamma*I', nrestart)
      fprintf('\n gamma =%9.2e      gammasum =%9.2e\n', gamma,gammasum)
      x0       = xminres2;

      [ xminres2, istop, itn, rnorm, Arnorm, Anorm, Acond, xnorm, betacheck ] = ...
          minres( x0, A, b, M, precon, shift, show, check, itnlim, rtol, tolMPD );

      itntotal = itntotal + itn;
   end
   TSminres2 = toc;
   fprintf('\n nrestart =%3i    gammasum =%8.1e    itntotal =%7i\n', ...
               nrestart,        gammasum,          itntotal)
   fprintf(' Time for minres with Mminres2 =%9.2e\n', TSminres2)

end % function minresMminres2
