function [DAD,D] = scaleSID( A,iprint )

% iprint = 1;   [DAD,D] = scaleSID(A,iprint);
% assumes A is a sparse symmetric matrix (possibly indefinite).
% It returns DAD and diagonal sparse D such that DAD = D*A*D is
% exactly symmetric and max |DAD(i,j)| = 1.
% If iprint>0, the min and max scales in D are printed.

% 06 May 2020: First version of scaleSID.
%              Shaked Regev and Michael Saunders, ICME, Stanford University.

  [m,n] = size(A);
  if iprint
     fprintf('\n scaleSID: Symmetric scaling of SID matrix.   n =%9i   nnz =%9i', n,nnz(A))
  end
  if m ~= n, disp('A must be square'); return; end

  scale = max(abs(A));             % sparse row vector
  scale = full(scale');            % dense  column vector
  Iz    = find(scale==0);
  lenz  = length(Iz);
  if lenz > 0
    disp(['A  contains ', num2str(lenz), ' empty columns'])
    scale(Iz) = 1;
  end

  d     = full(diag(A));           % dense  column vector
  scale = 1./scale;                % dense  column vector
  d     = scale.*d;                % dense  diagonal of DAD
  scale = sqrt(scale);             % dense  elements of D
  D     = diag(sparse(scale));     % sparse diagonal D
  A     = D*tril(A,-1)*D;
  DAD   = A + A' + diag(sparse(d));

  if iprint
    [smin,jmin] = min(scale);
    [smax,jmax] = max(scale);
    fprintf('\n\n   Min scale                Max scale')
    fprintf('\n  Col %7g %9.1e    Col %7g %9.1e\n', jmin, smin, jmax, smax)
  end

end % function scaleSID
