function testSPD( rtol );

% rtol = 1e-12;   testSPD( rtol );
% testSPD  tests MINRES with the SSAI preconditioner on bcsstk13.mat
% with specified residual tolerance rtol.
% bcsstk13 is an ill-conditioned test problem.
% condest(A) = 4.6e+10 before scaling
%            = 3.3e+06 after scaling.

% 16 Jun 2020: First version of testSPD for minres20
%              to facilitate a quick check.

  load bcsstk13.mat;

% Extract A = Problem.A from symmetric sparse matrix in SuiteSparse format,
% scale A, and create b such that Ax = b has a known solution (xtrue).
% xtrue(1) = 1 and MINRES prints x(1).  This allows us to see if MINRES
% is close to converging.
% (rtol, Anorm, bnorm, rnorm, xnorm are used to terminate MINRES.)

% SPD = 1;   [A,D,b,xtrue] = makeAbx( Problem,SPD );

  A      = Problem.A;
  n      = length(A);
  xtrue  = (n:-1:1)'/n;   % xtrue(1) = 1
  b      = A*xtrue;

  iprint = 1;
  [DAD,D]= scaleSPD(A,iprint);
  Db     = D*b;

  xSSAI  = minresSSAI( DAD,Db,rtol );
  x      = D*xSSAI;
  enorm  = norm(x-xtrue,inf);

  fprintf('\n rtol = %9.1e    Inf-norm of error in x = %9.1e\n', rtol, enorm)
end % function testSPD
